/*
 * Copyright 2004 The Apache Software Foundation.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * $Header:$
 */
package org.apache.beehive.netui.compiler.model;

import org.apache.beehive.netui.compiler.model.schema.struts11.SetPropertyDocument.SetProperty;
import org.apache.beehive.netui.compiler.model.schema.struts11.ExceptionDocument;
import org.apache.beehive.netui.compiler.model.validation.ValidatorConstants;
import org.apache.beehive.netui.compiler.JpfLanguageConstants;

public class ExceptionModel
        extends StrutsElementSupport
        implements JpfLanguageConstants
{
    private String _type;
    private String _path;
    private String _handlerMethod;
    private String _message;
    private String _messageKey;
    private String _handlerClass;
    private boolean _readonly = false;

    private static final String JPF_EXCEPTION_CONFIG_CLASSNAME = PAGEFLOW_PACKAGE + ".config.PageFlowExceptionConfig";
    
    
    protected ExceptionModel( StrutsApp parentApp )
    {
        super( parentApp );
    }
    
    public ExceptionModel( String type, String path, String handlerMethod, String message,
                           String messageKey, StrutsApp parentApp )
    {
        super( parentApp );
        
        _type = type;
        _path = path;
        _handlerMethod = handlerMethod;
        _message = message;
        _messageKey = messageKey;
    }

    public String getType()
    {
        return _type;
    }

    public void setType( String type )
    {
        _type = type;
    }

    public String getPath()
    {
        return _path;
    }

    public void setPath( String path )
    {
        _path = path;
    }

    public String getHandlerMethod()
    {
        return _handlerMethod;
    }

    public void setHandlerMethod( String handlerMethod )
    {
        _handlerMethod = handlerMethod;
    }

    public String getMessage()
    {
        return _message;
    }

    public void setMessage( String message )
    {
        _message = message;
    }

    public String getMessageKey()
    {
        return _messageKey;
    }

    public void setMessageKey( String messageKey )
    {
        _messageKey = messageKey;
    }

    public String getHandlerClass()
    {
        return _handlerClass;
    }

    public void setHandlerClass( String handlerClass )
    {
        _handlerClass = handlerClass;
    }

    public void writeToXMLBean( ExceptionDocument.Exception xb )

    {
        xb.setType( _type );

        if ( xb.getPath() == null && _path != null )
        {
            boolean relativeToModule = ! _path.startsWith( "/" );
            
            if ( relativeToModule )
            {
                xb.setPath( "/" + _path );  // struts wants "/" -- assumes this is module-relative path
            }
            else
            {
                xb.setPath( _path );
                addSetProperty( xb, "isPathContextRelative", "true" );
            }
        }
        
        if ( xb.getKey() == null && _messageKey != null ) xb.setKey( _messageKey );
        
        //
        // Struts doesn't support "message" directly -- we'll add this as a custom property.
        //
        if ( _message != null ) addSetProperty( xb, "defaultMessage", _message );
        
        if ( xb.getKey() == null ) xb.setKey( _type );
        
        //
        // Note that we're setting the handler *method* as the handler.  This would break Struts.
        //
        if ( xb.getHandler() == null && _handlerMethod != null && _handlerClass == null )
        {
            xb.setHandler( _handlerMethod );
            addSetProperty( xb, "isHandlerMethod", "true" );
        }
        
        if ( _readonly ) addSetProperty( xb, "readonly", "true" );
        if ( xb.getHandler() == null && _handlerClass != null ) xb.setHandler( _handlerClass );
    }
    
    private void addSetProperty( ExceptionDocument.Exception xb, String propertyName, String propertyValue )
    {
        SetProperty prop = xb.addNewSetProperty();
        prop.setProperty( propertyName );
        prop.setValue( propertyValue );
        if ( xb.getClassName() == null ) xb.setClassName( JPF_EXCEPTION_CONFIG_CLASSNAME );
    }

    public boolean isReadonly()
    {
        return _readonly;
    }

    public void setReadonly( boolean readonly )
    {
        _readonly = readonly;
    }
}
