/*
 * Copyright 2004 The Apache Software Foundation.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * $Header:$
 */
package org.apache.beehive.netui.pageflow;

import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;

import javax.servlet.ServletContext;
import javax.servlet.ServletResponse;
import javax.servlet.ServletRequest;


/**
 * An event reporter, which will be notified of events like "page flow created", "action raised", etc.
 */
public abstract class PageFlowEventReporter
{
    private ServletContext _servletContext;
    
    protected PageFlowEventReporter( ServletContext servletContext )
    {
        _servletContext = servletContext;
    }
    
    public abstract void actionRaised( FlowController flowController, ActionMapping mapping, ActionForm form,
                                       ServletRequest request, ServletResponse response );
    
    public abstract void actionSuccess( FlowController flowController, ActionMapping mapping, ActionForm form,
                                        ServletRequest request, ServletResponse response, ActionForward result,
                                        long timeTakenMillis );
    
    public abstract void exceptionRaised( Throwable ex, ActionMapping actionMapping, ActionForm form,
                                          FlowController flowController, ServletRequest request,
                                          ServletResponse response );
    
    public abstract void exceptionHandled( Throwable ex, ActionMapping actionMapping, ActionForm form,
                                           FlowController flowController, ServletRequest request,
                                           ServletResponse response, ActionForward result,
                                           long timeTakenMillis );
    
    public abstract void flowControllerCreated( FlowController flowController, ServletRequest request,
                                                ServletResponse response );
    
    public abstract void flowControllerDestroyed( FlowController flowController );
    
    public abstract void beginActionRequest( ServletRequest request, ServletResponse response );
    
    public abstract void endActionRequest( ServletRequest request, ServletResponse response, long timeTakenMillis );
    
    public abstract void beginPageRequest( ServletRequest request, ServletResponse response );
    
    public abstract void endPageRequest( ServletRequest request, ServletResponse response, long timeTakenMillis );
    
    protected ServletContext getServletContext()
    {
        return _servletContext;
    }
}
