/*
 * Copyright 2004 The Apache Software Foundation.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * $Header:$
 */
package org.apache.beehive.netui.pageflow.config;

import org.apache.struts.config.ControllerConfig;
import org.apache.struts.upload.CommonsMultipartRequestHandler;
import org.apache.struts.upload.DiskMultipartRequestHandler;
import org.apache.beehive.netui.util.config.bean.PageflowConfig;
import org.apache.beehive.netui.pageflow.internal.InternalUtils;

import java.util.LinkedHashMap;
import java.util.Map;

/**
 * Bean class to handle our extensions to the Struts &lt;controller&gt; element.
 */
public class PageFlowControllerConfig extends ControllerConfig
{
    private static final String MEMORY_MULTIPART_HANDLER_CLASS = CommonsMultipartRequestHandler.class.getName();
    private static final String DISK_MULTIPART_HANDLER_CLASS = DiskMultipartRequestHandler.class.getName();
    
    private boolean _isNestedPageFlow;
    private boolean _isLongLivedPageFlow;
    private boolean _isReturnToPageDisabled;
    private boolean _isReturnToActionDisabled;
    private boolean _isMissingDefaultMessages;
    private LinkedHashMap/*< String, String >*/ _sharedFlowTypes;
    private String _controllerClass;
    private boolean _isSharedFlow;
    private String _overrideMultipartClass = null;
    private boolean _forceMultipartDisabled = false;

    
    public boolean isNestedPageFlow()
    {
        return _isNestedPageFlow;
    }

    public void setIsNestedPageFlow( boolean nestedPageFlow )
    {
        _isNestedPageFlow = nestedPageFlow;
    }

    public boolean isLongLivedPageFlow()
    {
        return _isLongLivedPageFlow;
    }

    public void setIsLongLivedPageFlow( boolean longLivedPageFlow )
    {
        _isLongLivedPageFlow = longLivedPageFlow;
    }

    public boolean isReturnToPageDisabled()
    {
        return _isReturnToPageDisabled;
    }

    public void setIsReturnToPageDisabled( boolean returnToPageDisabled )
    {
        _isReturnToPageDisabled = returnToPageDisabled;
    }

    public boolean isReturnToActionDisabled()
    {
        return _isReturnToActionDisabled;
    }

    public void setIsReturnToActionDisabled( boolean returnToActionDisabled )
    {
        _isReturnToActionDisabled = returnToActionDisabled;
    }

    public boolean isMissingDefaultMessages()
    {
        return _isMissingDefaultMessages;
    }

    public void setIsMissingDefaultMessages( boolean missingDefaultMessages )
    {
        _isMissingDefaultMessages = missingDefaultMessages;
    }

    public void setSharedFlows( String sharedFlows )
    {
        if ( sharedFlows == null || sharedFlows.length() == 0 )
        {
            _sharedFlowTypes = null;
            return;
        }
        
        String[] keyValues = sharedFlows.split( "," );
        _sharedFlowTypes = new LinkedHashMap/*< String, String >*/();
        
        for ( int i = 0; i < keyValues.length; i++ )
        {
            String keyValue = keyValues[i];
            int delim = keyValue.indexOf( '=' );
            assert delim != -1 : "no delimiter in " + keyValue;
            assert delim < keyValue.length() - 1 : "missing value in " + keyValue;
            _sharedFlowTypes.put( keyValue.substring( 0, delim ), keyValue.substring( delim + 1 ) );
        }
    }
    
    public String getSharedFlows()
    {
        throw new UnsupportedOperationException( "not implemented; uses getSharedFlowTypes" );
    }
    
    public Map/*< String, String >*/ getSharedFlowTypes()
    {
        return _sharedFlowTypes;
    }

    public String getControllerClass()
    {
        return _controllerClass;
    }

    public void setControllerClass( String controllerClass )
    {
        _controllerClass = controllerClass;
    }

    public boolean isSharedFlow()
    {
        return _isSharedFlow;
    }

    public void setIsSharedFlow( boolean sharedFlow )
    {
        _isSharedFlow = sharedFlow;
    }

    public String getMultipartClass()
    {
        if ( _forceMultipartDisabled ) return null;
        if ( _overrideMultipartClass != null ) return _overrideMultipartClass;
        
        PageflowConfig.MultipartHandler.Enum mpHandler = InternalUtils.getMultipartHandlerType();
        
        if ( mpHandler != null )
        {
            switch ( mpHandler.intValue() )
            {
                case PageflowConfig.MultipartHandler.INT_DISABLED: return null;
                case PageflowConfig.MultipartHandler.INT_MEMORY: return MEMORY_MULTIPART_HANDLER_CLASS;
                case PageflowConfig.MultipartHandler.INT_DISK: return DISK_MULTIPART_HANDLER_CLASS;
                default: assert false : "unknown value for multipart handler: " + mpHandler.toString();
            }
        }
        
        return null;
    }

    /**
     * Get the directory for DiskMultipartRequestHandler temporary files.
     * @return directory string
     */ 
    public String getTempDir()
    {
        // TODO: In the Struts 1.1 version of DiskMultipartRequestHandler, there is an NPE on line 295
        // (in retrieveTempDir).  When this is fixed, we can remove our override of getTempDir().
        return tempDir != null ? tempDir : System.getProperty( "java.io.tmpdir" );
    }

    public void setMultipartClass( String overrideMultipartClass )
    {
        if ( overrideMultipartClass.equals( "none" ) )
        {
            _forceMultipartDisabled = true;
        }
        else
        {
            _overrideMultipartClass = overrideMultipartClass;
        }
    }
}
