/*
 * Copyright 2004 The Apache Software Foundation.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * $Header:$
 */
package org.apache.beehive.netui.pageflow.interceptor.action;

import org.apache.beehive.netui.util.logging.Logger;
import org.apache.beehive.netui.pageflow.Forward;

import javax.servlet.http.HttpServletRequest;
import java.util.Enumeration;
import java.util.Map;
import java.util.HashMap;
import java.util.Iterator;
import java.io.Serializable;

class OriginalForward extends InterceptorForward
{
    private Map _savedAttrs;
    
    private static final Logger _log = Logger.getInstance( OriginalForward.class );
    
    
    OriginalForward( Forward fwd, HttpServletRequest request )
    {
        super( fwd );
        saveRequestAttrs( request );
    }
    
    OriginalForward( HttpServletRequest request )
    {
        super( request );
        saveRequestAttrs( request );
    }
    
    private void saveRequestAttrs( HttpServletRequest request )
    {
        _savedAttrs = new HashMap();
        
        for ( Enumeration e = request.getAttributeNames(); e.hasMoreElements(); )
        {
            String attrName = ( String ) e.nextElement();
            Object attrVal = request.getAttribute( attrName ); 
            
            if ( attrVal instanceof Serializable )
            {
                _savedAttrs.put( attrName, attrVal );
            }
            else
            {
                if ( _log.isWarnEnabled() )
                {
                    _log.warn( "Dropping non-serializable request attribute " + attrName + " (" + attrVal + ")." );
                }
            }
        } 
    }
    
    public void rehydrateRequest( HttpServletRequest request )
    {
        if ( _savedAttrs != null )
        {
            for ( Iterator i = _savedAttrs.entrySet().iterator(); i.hasNext(); )
            {
                Map.Entry entry = ( Map.Entry ) i.next();
                
                String attrName = ( String ) entry.getKey();
                if ( request.getAttribute( attrName ) == null )
                {
                    request.setAttribute( attrName, entry.getValue() );
                }
            }
        }
    }
}
