/*
 * Copyright 2004 The Apache Software Foundation.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * $Header:$
 */
package org.apache.beehive.netui.tags.databinding.xml;

import javax.servlet.jsp.JspException;

import org.apache.beehive.netui.tags.AbstractClassicTag;
import org.apache.beehive.netui.util.internal.InternalStringBuilder;
import org.apache.beehive.netui.util.logging.Logger;
import org.apache.xmlbeans.XmlObject;

/**
 * <p>
 * A tag that serializes an XMLBean into the output of a JSP page in order to move data to the browser for data binding.
 * </p>
 * @jsptagref.tagdescription
 * A tag that serializes an XMLBean into the output of a JSP page in order to move data to the browser for data binding.
 * @netui:tag name="serializeXML"
 *            description="A tag that serializes an XMLBean into the output of a JSP page in order to move data to the browser for data binding."
 */
public class SerializeXML
        extends AbstractClassicTag {

    private static final Logger LOGGER = Logger.getInstance(SerializeXML.class);

    private Object _source;
    private String _divName;

    /**
     * The name of this tag; this value is used for error reporting.
     * @return the String name of this tag
     */
    public String getTagName() {
        return "SerializeXML";
    }

    /**
     * Set the source document to be serialized.  The object referenced here should be an instance of
     * {@link XmlObject}.
     * @param source the source
     * @jsptagref.attributedescription
     * Set the source document to be serialized.  The object referenced here should be an instance of
     * {@link XmlObject}.
     * @jsptagref.attributesyntaxvalue <i>object_source</i>
     * @netui:attribute required="true" rtexprvalue="true"
     */
    public void setSource(Object source) {
        _source = source;
    }

    /**
     * Set the name of the div into which this XML will be rendered.
     * @param divName the div name
     * @jsptagref.attributedescription
     * Set the name of the div into which this XML will be rendered.
     * @jsptagref.attributesyntaxvalue <i>string_divName</i>
     * @netui:attribute required="true" rtexprvalue="true"
     */
    public void setDivName(String divName) {
        _divName = divName;
    }

    public int doStartTag() {
        return SKIP_BODY;
    }

    public int doEndTag()
        throws JspException {

        if(_source instanceof XmlObject && _source != null) {
            XmlObject xmlObject = (XmlObject)_source;
            String xml = xmlObject.xmlText();

            InternalStringBuilder buf = new InternalStringBuilder();
            buf.append("<div");
            buf.append(" id=\"");
            buf.append(_divName);
            buf.append("\">\n<!--\n");
            buf.append(xml);
            buf.append("\n-->\n</div>");

            write(buf.toString());
        }
        else {
            // @todo: write anything to the page?
            if(LOGGER.isInfoEnabled())
                LOGGER.info("The expression \"" + _source + "\" resulted in an object that was not an XMLBean");
        }

        localRelease();

        return EVAL_PAGE;
    }

    public void localRelease() {
        super.localRelease();

        _source = null;
        _divName = null;
    }
} 
