/*
 * Copyright 2004 The Apache Software Foundation.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * $Header:$
 */

package org.apache.beehive.netui.tools.testrecorder.shared;

import java.io.PrintWriter;
import java.io.StringWriter;

import org.apache.log4j.*;
import org.apache.commons.logging.Log;

public class Logger implements Log {

    private static final String STRUTS_APPENDER = "commons-logging";

    static {
        // Need to get rid of the appender that Struts adds so
        // that we don't spam the console with all messages
        Category root = Category.getRoot();

        if ( root.getAppender( STRUTS_APPENDER ) != null ) {
            root.removeAppender( STRUTS_APPENDER );
        }
    }


    public static Logger getInstance( Class clazz ) {
        return new Logger( clazz );
    }

    private org.apache.log4j.Logger logInstance;

    public Logger( Class clazz ) {
        logInstance = org.apache.log4j.Logger.getLogger( clazz.getName() );
    }

    public boolean isDebugEnabled() {
        return logInstance.isEnabledFor( Level.DEBUG );
    }

    public boolean isErrorEnabled() {
        return logInstance.isEnabledFor( Level.ERROR );
    }

    public boolean isFatalEnabled() {
        return logInstance.isEnabledFor( Level.FATAL );
    }

    public boolean isInfoEnabled() {
        return logInstance.isEnabledFor( Level.INFO );
    }

    public boolean isTraceEnabled() {
        return logInstance.isEnabledFor( Level.DEBUG );
    }

    public boolean isWarnEnabled() {
        return logInstance.isEnabledFor( Level.WARN );
    }

    public void debug( Object message ) {
        if ( logInstance.isEnabledFor( Level.DEBUG ) ) {
            logInstance.debug( message );
        }
    }

    public void debug( Object message, Throwable t ) {
        if ( logInstance.isEnabledFor( Level.DEBUG ) ) {
            logInstance.debug( format( message, t ) );
        }
    }

    public void trace( Object message ) {
        if ( logInstance.isEnabledFor( Level.DEBUG ) ) {
            logInstance.debug( message );
        }
    }

    public void trace( Object message, Throwable t ) {
        if ( logInstance.isEnabledFor( Level.DEBUG ) ) {
            logInstance.debug( format( message, t ) );
        }
    }

    public void info( Object message ) {
        if ( logInstance.isEnabledFor( Level.INFO ) ) {
            logInstance.info( message );
        }
    }

    public void info( Object message, Throwable t ) {
        if ( logInstance.isEnabledFor( Level.INFO ) ) {
            logInstance.info( format( message, t ) );
        }
    }

    public void warn( Object message ) {
        if ( logInstance.isEnabledFor( Level.WARN ) ) {
            logInstance.warn( message );
        }
    }

    public void warn( Object message, Throwable t ) {
        if ( logInstance.isEnabledFor( Level.WARN ) ) {
            logInstance.warn( format( message, t ) );
        }
    }

    public void error( Object message ) {
        if ( logInstance.isEnabledFor( Level.ERROR ) ) {
            logInstance.error( message );
        }
    }

    public void error( Object message, Throwable t ) {
        if ( logInstance.isEnabledFor( Level.ERROR ) ) {
            logInstance.error( format( message, t ) );
        }
    }

    public void fatal( Object message ) {
        if ( logInstance.isEnabledFor( Level.FATAL ) ) {
            logInstance.fatal( message );
        }
    }

    public void fatal( Object message, Throwable t ) {
        if ( logInstance.isEnabledFor( Level.FATAL ) ) {
            logInstance.fatal( format( message, t ) );
        }
    }

    public static String format( final Object obj, final Throwable t ) {
        if ( t == null ) {
            return obj.toString();
        }
        Throwable ex = t;
        StringBuffer sb = new StringBuffer( 512 );
        sb.append( "object( " + obj + " )\n" );
        while ( ex != null ) {
            sb.append( format( ex ) + "\n");
            ex = ex.getCause();
        }
        return sb.toString();
    }

    public static String format( Throwable t ) {
        if ( t == null ) {
            return "NULL";
        }
        StringWriter sw = new StringWriter();
        t.printStackTrace( new PrintWriter( sw ) );
        return "throwable: msg( " + t.getMessage() + " )\ntoString( " + t.toString() +
                " )\nstack trace:\n" + sw.toString();
    }

}
