/*
 * Copyright 2004 The Apache Software Foundation.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * $Header:$
 */

package org.apache.beehive.netui.tools.testrecorder.shared;

import java.util.List;
import java.util.ArrayList;
import java.util.Collections;

/**
 */
public class RecordSessionBean extends SessionBean {

    private List requestData;
    private List responseData;
    // if true indicates an error has occured in the session
    protected boolean error = false;

    public RecordSessionBean( String sessionName ) {
        this.sessionName = sessionName;
        requestData = new ArrayList();
        responseData = new ArrayList();
    }

    public void setError() {
        error = true;
    }

    public void setError( boolean error ) {
        this.error = error;
    }

    public boolean isError() {
        return error;
    }


    public List getRequestData() {
        return Collections.unmodifiableList( requestData );
    }

    /**
     *
     * @param index
     * @return
     * @throws IndexOutOfBoundsException
     */
    public RequestData getRequestData( int index ) {
        return (RequestData) requestData.get( index );
    }

    public List getResponseData() {
        return Collections.unmodifiableList( responseData );
    }

    /**
     * @param index
     * @return
     * @throws IndexOutOfBoundsException
     */
    public ResponseData getResponseData( int index ) {
        return (ResponseData) responseData.get( index );
    }

    public void addRequestResponseData( RequestData request, ResponseData response ) {
        requestData.add( request );
        responseData.add( response );
    }

    /**
     * update the data for the most recent test
     * @param request
     * @param response
     */
    public void updateRequestResponseData( RequestData request, ResponseData response ) {
        if ( requestData.size() == 0 || responseData.size() == 0 ) {
            throw new IllegalStateException( "data size is zero, unable to update");
        }
        requestData.set( requestData.size() -1, request );
        responseData.add( responseData.size() -1, response );
    }

    public int getTestCount() {
        return requestData.size();
    }

    public String toString() {
        StringBuffer sb = new StringBuffer( 256 );
        sb.append( "[ " );
        sb.append( "sessionName( " + getSessionName() + " )" );
        sb.append( ", tester( " + getTester() + " )" );
        sb.append( ", startDate( " + getStartDateString() + " )" );
        sb.append( ", endDate( " + getEndDateString() + " )" );
        sb.append( ", testCount( " + getTestCount() + " )" );
        sb.append( ", description( " + getDescription() + " )" );
        sb.append( " ]" );
        return sb.toString();
    }

}
