/*
 * Copyright 2004 The Apache Software Foundation.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * $Header:$
 */

package org.apache.beehive.netui.tools.testrecorder.shared;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;
import java.util.Iterator;
import java.util.Enumeration;
import java.util.Map;
import java.util.Set;
import java.util.List;
import java.util.ArrayList;

public class Util {

    static final int REPORT_SESSION = 0;
    static final int REPORT_REQUEST = 1;

    private static final String LT = "<";
    private static final String LT_END = "</";
    private static final String GT = ">";

    private Util() {
    }

    public static List getListOfKeys( Map map ) {
        List keys = null;
        if ( map != null ) {
            Set keySet = map.keySet();
            if ( keySet.size() > 0 ) {
                keys = new ArrayList( keySet.size() );
                keys.addAll( keySet );
            }
        }
        if ( keys == null ) {
            keys = new ArrayList();
        }
        return keys;
    }

    public static String toString( Map map ) {
        Set keySet = null;
        StringBuffer sb = new StringBuffer();

        if ( map != null ) {
            keySet = map.keySet();
            sb.append( "Map(" + keySet.size() + ")[ " );
        }
        else {
            sb.append( "Map[ " );
        }

        if ( map != null ) {
            Iterator it = keySet.iterator();
            Object key = null;
            int i = 0;
            for ( i = 0; it.hasNext(); i++ ) {
                key = it.next();
                if ( i != 0 ) {
                    sb.append( "; " );
                }
                sb.append( "key(" + key + "), value( " + map.get( key ) + " )" );
            }
            if ( i == 0 ) {
                sb.append( "empty" );
            }
        }
        else {
            sb.append( "null" );
        }

        sb.append( " ]" );
        return sb.toString();
    }

    public static String toString( Object[] objects ) {
        if ( objects == null ) {
            return null;
        }
        StringBuffer sb = new StringBuffer( 16 * objects.length );
        sb.append( "[ " );
        int i = 0;
        for ( i = 0; i < objects.length; i++ ) {
            sb.append( ", [" + i + "]( " + objects[i] + " )" );
        }
        if ( i == 0 ) {
            sb.append( "EMPTY" );
        }
        sb.append( " ]" );
        return sb.toString();
    }

    public static String toString( Iterator it, int size ) {
        StringBuffer sb = new StringBuffer( 16 * size );
        sb.append( "[ " );
        int i = 0;
        for ( i = 0; it.hasNext(); i++ ) {
            sb.append( ", [" + i + "]( " + it.next() + " )" );
        }
        if ( i == 0 ) {
            sb.append( "EMPTY" );
        }
        sb.append( " ]" );
        return sb.toString();
    }

    public static final String genStartTag( final String tagName ) {
        return LT + tagName + GT;
    }

    public static final String genStartTag( final String tagName, final String attributes ) {
        if ( attributes == null ) {
            return genStartTag( tagName );
        }
        return LT + tagName + " " + attributes + GT;
    }

    public static final String genEndTag( final String tagName ) {
        return LT_END + tagName + GT;
    }

    public static StringBuffer reportSessionAttributes( StringBuffer sb,
            HttpServletRequest request, String id, String separator ) {
        return reportAttributes( sb, request, id, separator, REPORT_SESSION );
    }

    public static StringBuffer reportRequestAttributes( StringBuffer sb,
            HttpServletRequest request, String id, String separator ) {
        return reportAttributes( sb, request, id, separator, REPORT_REQUEST );
    }

    static StringBuffer reportAttributes( StringBuffer sb,
            HttpServletRequest request, String id, String separator, int type ) {
        HttpSession session = request.getSession();
        Enumeration e = session.getAttributeNames();
        String key = null;
        Object value = null;
        while ( e.hasMoreElements() ) {
            key = (String) e.nextElement();
            switch ( type ) {
                case REPORT_REQUEST:
                    value = request.getAttribute( key );
                    break;
                case REPORT_SESSION:
                    value = request.getAttribute( key );
                    break;
                default:
                    throw new RuntimeException();
            }
            reportAttribute( sb, key, value, id, separator );
        }
        return sb;
    }

    public static StringBuffer reportAttribute( StringBuffer sb, String key,
            Object value, String id, String separator ) {
        sb.append( id + " [ key( " + key + " ), value(" + value.getClass().getName() + ")( " +
                value + " ) ]" + separator );
        return sb;
    }

}
