package org.apache.beehive.wsm.axis.security.model;

/*
 * DropInDeploymentHandler.java
 *
 * Copyright 2001-2004 The Apache Software Foundation.
 *
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 */

import java.util.Collection;
import java.util.Hashtable;
import java.util.Map;

import org.apache.commons.codec.digest.DigestUtils; 

import org.apache.beehive.wsm.axis.security.User;
import org.apache.beehive.wsm.axis.security.Group;
import org.apache.beehive.wsm.axis.security.UserList;
import org.apache.beehive.wsm.axis.security.Role;


public class MemoryUserImpl implements User {

    private String name;
    private String password;
    private boolean md5;

    private Map<String,Role> roles;
    private Map<String,Group> groups;

    public MemoryUserImpl ()
    {
        roles = new Hashtable<String,Role>();
        groups = new Hashtable<String,Group>();
    }

    public void setName ( String name )
    {
        this.name = name;
    }

    public String getName ()
    {
        return name;
    }

    public void setPassword ( String password )
    {
        this.password = password;
    }

    public String getPassword ()
    {
        return password;
    }

    public void setMd5 ( boolean isMd5 )
    {
        this.md5 = isMd5;
    }

    public boolean isMd5 ()
    {
        return md5;
    }

    public boolean authenticate ( String password )
    {
        try{
            if ( ( this.password == null ) && ( password == null ) ){
                // if both null, returns true;   Probably a user intentionally set null for both.
                return true;
            }

            if ( isMd5() )
            {
                // todo: implement in case md5 is used.
                if ( this.password.equals( DigestUtils.md5Hex(password))  )
                    return true;
            }
            else
            {
                if ( this.password.equals( password ) )
                    return true;
            }
        }catch(NullPointerException npe){
            return false;
        }

        return false;

    }

    public void addGroup( Group group )
    {
        groups.put(group.getName(), group);
    }

    public Group getGroup ( String group )
    {
        return groups.get(group);
    }

    public Collection<Group> getGroups ()
    {
        return groups.values();
    }

    public void addRole( Role role )
    {
        roles.put(role.getName(), role);
    }

    public Role getRole ( String role )
    {
        return roles.get(role);
    }

    public Collection<Role> getRoles ()
    {
        return roles.values();
    }

}
