/*
 * 
 * Copyright 2001-2004 The Apache Software Foundation.
 * 
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 * 
 */
package org.apache.beehive.wsm.databinding.xmlbeans;

import java.lang.reflect.Method;

import javax.xml.namespace.QName;

import org.apache.beehive.wsm.databinding.BindingLookupService;
import org.apache.log4j.Logger;
import org.apache.xmlbeans.SchemaField;
import org.apache.xmlbeans.SchemaType;
import org.apache.xmlbeans.SchemaTypeLoader;
import org.apache.xmlbeans.XmlBeans;
import org.apache.xmlbeans.XmlObject;



public class XmlBeanTypeLookup implements BindingLookupService {
	static Logger logger = Logger.getLogger(XmlBeanTypeLookup.class);
	static XmlBeanTypeLookup theLookup = new XmlBeanTypeLookup();
    
	public QName class2qname( Class cls) {
	    if (XmlObject.class.isAssignableFrom(cls)) {
            return XmlBeans.typeForClass(cls).getName();
        }
	    return null;
	}
    
    public QName class2qname(Class cls, String defaultnamespace) {
        QName res = class2qname(cls);
        // for xmlbeans we can ignore default name space.
        return res;
    }

	/* (non-Javadoc)
	 * @see TypeLookUpServices#Qname2Class(javax.xml.namespace.QName)
	 */
	public Class qname2class(QName qname) {
		logger.debug("Get XMLBeans class for Qname: " + qname);
	        SchemaTypeLoader stl = XmlBeans.getContextTypeLoader();
	        SchemaType st = stl.findType(qname);
	        if (st == null) {
	            SchemaField sf = stl.findElement(qname);
	            if (sf != null) {
	                st = sf.getType();
	            }
	        }

	        if (st != null) {
	            Class xmlClass = st.getJavaClass();

	            //String clName = xmlClass.getName();
	            if (st.isBuiltinType()) {
	                Method[] declared = xmlClass.getDeclaredMethods();
	                Class natural =
	                    scanDeclaredMethodsForViableReturnType(declared);
	                if (natural != null) {
	                    return natural;
	                }
	                else {
	                    // NOTE jcolwell@bea.com 2004-Nov-12 --
	                    // XmlString declares no methods

	                    if (xmlClass.isInterface()) {
	                        for (Class cl : xmlClass.getInterfaces()) {
	                            natural = scanDeclaredMethodsForViableReturnType
	                                (cl.getDeclaredMethods());
	                            if (natural != null) {
	                                return natural;
	                            }
	                        }
	                    }
	                    else {
	                        declared = xmlClass.getSuperclass().getDeclaredMethods();
	                        natural = scanDeclaredMethodsForViableReturnType
	                            (declared);
	                        if (natural != null) {
	                            return natural;
	                        }
	                    }
	                }
	            }
	            return xmlClass;
	        }
	        else {
	            // NOTE jcolwell@bea.com 2004-Nov-30 -- 
	            // keep in mind that a real TMU based on a viable SOAP stack should
	            // be used and this pure XmlBean implementation is just a fallback.
	            return null;
	        }
	    }
	
	   private Class scanDeclaredMethodsForViableReturnType
       (Method[] declared) {
       /* TODO:  Does this make sense?  IT looks as if the type of the class is determined based on the
        * method return types.  
        */
       for (Method meth : declared) {
               
           Class returnType = meth.getReturnType();
           //System.out.println(returnType.getName());
           if (!returnType.equals(Void.TYPE)) {
               /*
                * NOTE jcolwell@bea.com 2004-Nov-12 -- 
                * built-in XmlBeans types may be of the following natural
                * types:
                * primitives, byte arrays, Strings, Calendars, BigIntegers
                * and BigDecimals
                */
               if (returnType.isArray()
                   || returnType.isPrimitive()
                   || returnType.equals(String.class)
                   || returnType.equals(QName.class)
                   || returnType.equals(java.util.Calendar.class)
                   || returnType.equals(java.math.BigDecimal.class)
                   || returnType.equals(java.math.BigInteger.class)) {

                   return returnType;
               }
           }
       }
       return null;
   }

    /**
     * @param q
     * @return Class
     */
    public static Class q2Class(QName q) {
        return theLookup.qname2class(q);
    }
	
}
