package org.apache.beehive.wsm.model.jsr181;

/*
 * Copyright 2004 The Apache Software Foundation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * $Header:$
 */

import java.lang.annotation.Annotation;
import java.util.Collection;

import javax.jws.WebParam;

import javax.xml.namespace.QName; // todo (mmerz@apache.org, 01/25/05): this needs to go...

import org.apache.beehive.wsm.model.BeehiveWsParameterMetadata;
import org.apache.beehive.wsm.model.ValidationException;
import org.apache.beehive.wsm.model.java.JavaParameterInfo;

public class Jsr181ParameterMetadataImpl implements BeehiveWsParameterMetadata, java.io.Serializable {
    
    private static final long serialVersionUID = 1L;

    private String wpName;
    private String wpTargetNamespace;
    private WebParam.Mode wpMode;
    private boolean wpHeader;
    private Class javaType;
    private QName mXMLType;


    public void validate() throws ValidationException {
        return; // no validation required
    }

    public Jsr181ParameterMetadataImpl() {
    }

    // defaultName is argument name in the source file. It can be null if the
    // name is not known.
    public Jsr181ParameterMetadataImpl(JavaParameterInfo jp) {

        // check input params
        if (null == jp) {
            jp.logError("illegal java parameter info: <null>");
            return;
        }
        
        javaType = jp.getType();
        
        // set default values
        setWpName(jp.getName());
        setWpMode(WebParam.Mode.IN);
        
        // set optional annotations
        Collection<Annotation> annotations = jp.getAnnotations();
        if (null != annotations) {
            for (Annotation a : annotations) {
                if (a.annotationType() == WebParam.class) {
                    initFromAnnotation((WebParam) a);
                }
                else {
                    // todo: warning -- unknown annotation
                }
            }
        }
    }

    public boolean isWpHeader() {
        return wpHeader;
    }

    public void setWpHeader(boolean wpHeader) {
        this.wpHeader = wpHeader;
    }

    public WebParam.Mode getWpMode() {
        return wpMode;
    }

    public void setWpMode(WebParam.Mode wpMode) {
        this.wpMode = wpMode;
    }

    public String getWpName() {
        return wpName;
    }

    public void setWpName(String wpName) {
        this.wpName = wpName;
    }

    public String getWpTargetNamespace() {
         return wpTargetNamespace;
    }

    public void setWpTargetNamespace(String wpTargetNamespace) {
        this.wpTargetNamespace = wpTargetNamespace;
    }

    public Class getJavaType() {
        return javaType;
    }

    public void setJavaType(Class javaType) {
        this.javaType = javaType;
    }

    private void initFromAnnotation(WebParam annotation) {
        if (null != annotation) {
            if (0 < annotation.name().length()) {
                setWpName(annotation.name());
            }
            setWpTargetNamespace(annotation.targetNamespace());
            setWpMode(annotation.mode());
            setWpHeader(annotation.header());
        }
    }

    /* (non-Javadoc)
     * @see org.apache.beehive.wsm.jsr181.model.Jsr181ParameterMetadata#getJavaTypeFullName()
     * 
     * NOTE THIS METHOD AND SUPPORTING METHOD SHOULD BE MOVED OUT TO A ELEMENT CLASS
     * THAT ENCAPSULATES QNAME AND CLASS
     */
    public String getJavaTypeFullName() {
        return getClassName(getJavaType());

    }
    private String getClassName(Class cls) {
            if (cls.isArray()) {
                return getClassName(cls.getComponentType()) + "[]";
            }
            else {
                return cls.getName().replace('$', '.');
            }
    }
    
    public QName getXmlType() {
        return mXMLType;
    }

    public void setXmlType(QName xmlType) {
        mXMLType = xmlType;
    }
}
