package org.apache.beehive.wsm.model.jsr181;

import javax.jws.soap.SOAPBinding;

import org.apache.beehive.wsm.model.BeehiveWsSOAPBindingInfo;

/*
 * Copyright 2004 The Apache Software Foundation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * $Header:$
 */

public class SOAPBindingInfo implements java.io.Serializable, BeehiveWsSOAPBindingInfo {

    private static final long serialVersionUID = 1L;
    
    SOAPBinding.Style style = SOAPBinding.Style.DOCUMENT;
    SOAPBinding.Use use = SOAPBinding.Use.LITERAL;
    SOAPBinding.ParameterStyle parameterStyle = SOAPBinding
        .ParameterStyle.WRAPPED;

    /**
     * 
     */
    public SOAPBindingInfo() {
        super();
    }

    public SOAPBindingInfo(SOAPBinding annotation) {
        initFromAnnotation(annotation);
    }

    /**
     * @return Returns the parameterStyle.
     */
    public SOAPBinding.ParameterStyle getParameterStyle() {
        return parameterStyle;
    }

    /**
     * @param parameterStyle The parameterStyle to set.
     */
    public void setParameterStyle(SOAPBinding.ParameterStyle parameterStyle) {
        this.parameterStyle = parameterStyle;
    }

    /**
     * @return Returns the style.
     */
    public SOAPBinding.Style getStyle() {
        return style;
    }

    /**
     * @param style The style to set.
     */
    public void setStyle(SOAPBinding.Style style) {
        this.style = style;
    }

    /**
     * @return Returns the use.
     */
    public SOAPBinding.Use getUse() {
        return use;
    }

    /**
     * @param use The use to set.
     */
    public void setUse(SOAPBinding.Use use) {
        this.use = use;
    }

    /**
     * @param annotation
     */
    private void initFromAnnotation(SOAPBinding annotation) {
        setStyle(annotation.style());
        setUse(annotation.use());
        setParameterStyle(annotation.parameterStyle());
        
        // set the defaults -- no defaults to set
    }

    public int hashCode() {
        return (((getStyle().hashCode() * 31) + getUse().hashCode()) * 31) 
            + getParameterStyle().hashCode();
    }

    public String toString() {
        StringBuffer sb = new StringBuffer("SOAPBindingInfo: ");
        boolean rpc = (getStyle() == SOAPBinding.Style.RPC);
        boolean encoded = (getUse() == SOAPBinding.Use.ENCODED); 
        sb.append(rpc ? "RPC" : "DOCUMENT");
        sb.append('/');
        sb.append(encoded ? "ENCODED" : "LITERAL");
        if (!rpc && !encoded) {
            sb.append('-');
            sb.append((getParameterStyle() == SOAPBinding
                       .ParameterStyle.BARE) 
                      ? "BARE" : "WRAPPED");
        }
        return sb.toString();
    }

    public boolean equals(Object o) {
        if (o != null) {
            if (o instanceof SOAPBindingInfo) {
                SOAPBindingInfo sbi = (SOAPBindingInfo)o;
                return (getStyle().equals(sbi.getStyle())
                        && getUse().equals(sbi.getUse())
                        && getParameterStyle().equals(sbi.getParameterStyle()));
            }
        }
        return false;
    }
}
