/*
 * Copyright 2004 The Apache Software Foundation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *     http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * $Header:$
 */
package org.apache.beehive.wsm.model.jsr181;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

import javax.jws.soap.InitParam;
import javax.jws.soap.SOAPMessageHandler;

import org.apache.beehive.wsm.model.BeehiveWsSOAPMessageHandlerInfo;

public class SOAPMessageHandlerInfo implements java.io.Serializable, BeehiveWsSOAPMessageHandlerInfo {

    private static final long serialVersionUID = 1L;
    
    String name;
    String className;
    HashMap<String, String> parameterMap = new HashMap<String, String>();
    Collection<String> roles = new ArrayList<String>();
    Collection<String> headers = new ArrayList<String>();

    /**
     * 
     */
    public SOAPMessageHandlerInfo(SOAPMessageHandler annotation) {

        // validate className
        String className = annotation.className();
        if (0 == className.length()) {
            throw new IllegalArgumentException("className required to configure a SOAPMessageHandler");
        }
        
        // set fields from annotation
        setClassName(className);
        setName((0 < annotation.name().length()) ? annotation.name() : className);
        for (InitParam p : annotation.initParams()) {
            addInitParam(p.name(), p.value());
        }
        addRoles(Arrays.asList(annotation.roles()));
        for (String s : annotation.headers()) {
            addHeader(s);
        }
    }

    public SOAPMessageHandlerInfo(
        String handlerClassName,
        String handlerName,
        Map<String, String> initParams,
        Collection<String> roles,
        Collection<String> headers
    ) {
        setClassName(handlerClassName);
        setName((handlerName != null && handlerName.length() > 0)
                ? handlerName : handlerClassName);
        addInitParams(initParams);
        addRoles(roles);
        addHeaders(headers);
    }

    /**
     * @return Returns the className.
     */
    public String getClassName() {
        return className;
    }

    /**
     * @param className The className to set.
     */
    public void setClassName(String className) {
        this.className = className;
    }

    /**
     * @return Returns the name.
     */
    public String getName() {
        return name;
    }

    /**
     * @param name The name to set.
     */
    public void setName(String name) {
        this.name = name;
    }

    /**
     * @return Returns the headers.
     */
    public Collection<String> getHeaders() {
        return Collections.unmodifiableCollection(headers);
    }

    public void addHeader(String header) {
        this.headers.add(header);
    }

    /**
     * @param headers
     */
    private void addHeaders(Collection<String> headers) {
        this.headers.addAll(headers);
    }

    /**
     * @return Returns the params.
     */
    public Map<String, String> getParameterMap() {
        return Collections.unmodifiableMap(parameterMap);
    }

    public void addInitParam(String key, String value) {
        parameterMap.put(key, value);
    }

    public void addInitParams(Map<String, String> params) {
        parameterMap.putAll(params);
    }

    /**
     * @return Returns the roles.
     */
    public Collection<String> getRoles() {
        return Collections.unmodifiableCollection(roles);
    }

    public void addRole(String role) {
        this.roles.add(role);
    }

    public void addRoles(Collection<String> roles) {
        this.roles.addAll(roles);
    }

}
