package org.apache.beehive.wsm.processor.apt;

/*
 * Copyright 2004 The Apache Software Foundation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * $Header:$
 */

import java.util.Collection;

import org.apache.beehive.wsm.model.BeehiveWsSOAPBindingInfo;
import org.apache.beehive.wsm.model.BeehiveWsTypeMetadata;
import org.apache.beehive.wsm.model.BeehiveWsMethodMetadata;
import org.apache.beehive.wsm.model.BeehiveWsParameterMetadata;

public class WebServiceMetadataViewer {

    // adhock. assume Metadata is not nested more than 5 times.  :p
    private static String[] TABS = new String[]{
        "",
        "   ",
        "      ",
        "         ",
        "            ",
        "               "
    };

    public static void print(BeehiveWsTypeMetadata webServiceTYPEMetadata) {
        print(0, webServiceTYPEMetadata);
    }

    public static void print(int tab, BeehiveWsTypeMetadata webServiceTYPEMetadata) {
        print(tab, "<TYPE > > >WebServiceTYPEMetadata");
        print(tab, "wsName", webServiceTYPEMetadata.getWsName());
        print(tab, "wsServiceName", webServiceTYPEMetadata.getWsServiceName());
        print(tab, "wsWsdlLocation", webServiceTYPEMetadata.getWsWsdlLocation());
        print(tab, "wsTargetNamespace", webServiceTYPEMetadata.getWsTargetNamespace());
        print(tab, "wsEndpointInterface", webServiceTYPEMetadata.getWsEndpointInterface());
        print(tab, "hcFileName", webServiceTYPEMetadata.getHcFileName());
        print(tab, "hcName", webServiceTYPEMetadata.getHcName());
        //print( tab, "siRunAs"             , webServiceTYPEMetadata.getSiRunAs() );
//        print(tab, "serviceImplementationBean", webServiceTYPEMetadata.getServiceImplementationBean());
        print("");
        tab++;
        print(tab, webServiceTYPEMetadata.getMethods());
        print(tab, webServiceTYPEMetadata.getSoapBinding());
    }

    public static void print(Collection collection) {
        print(0, collection);
    }

    public static void print(int tab, Collection collection) {
        if (collection == null)
            return;
        for (Object element : collection) {
            if (element instanceof BeehiveWsTypeMetadata) {
                print(tab, (BeehiveWsTypeMetadata) element);
            } else if (element instanceof BeehiveWsMethodMetadata) {
                print(tab, (BeehiveWsMethodMetadata) element);
            } else if (element instanceof BeehiveWsParameterMetadata) {
                print(tab, (BeehiveWsParameterMetadata) element);
            }
        }
    }

    public static void print(BeehiveWsMethodMetadata webServiceMETHODMetadata) {
        print(0, webServiceMETHODMetadata);
    }

    public static void print(int tab, BeehiveWsMethodMetadata webServiceMETHODMetadata) {
        if (webServiceMETHODMetadata == null)
            return;
        print(tab, "<METHOD > > > > > >WebServiceMETHODMetadata");
        print(tab, "wmOperationName", webServiceMETHODMetadata.getWmOperationName());
        print(tab, "wmAction", webServiceMETHODMetadata.getWmAction());
        print(tab, "oneway", webServiceMETHODMetadata.isOneWay() ? "true" : "false");
        print(tab, "wrName", webServiceMETHODMetadata.getWrName());
        print(tab, "javaMethodName", webServiceMETHODMetadata.getJavaMethodName());
        print(tab, "javaReturnType", webServiceMETHODMetadata.getJavaReturnType());

        //print( tab, "DwRequestElement");
        //print( tab, webServiceMETHODMetadata.getDwRequestElement() );
        //print( tab, webServiceMETHODMetadata.getDwResponseElement() );

        //print( tab, "DwResponseElement");
        //print( tab, webServiceMETHODMetadata.getSoapBinding() );
 
        print("");
        tab++;
        print(tab, webServiceMETHODMetadata.getParams());
    }

    public static void print(BeehiveWsParameterMetadata webServicePARAMETERMetadata) {
        print(0, webServicePARAMETERMetadata);
    }

    public static void print(int tab, BeehiveWsParameterMetadata webServicePARAMETERMetadata) {
        if (webServicePARAMETERMetadata == null)
            return;
        print(tab, "<PARAM > > > > > > > > >WebServicePARAMETERMetadata");
        print(tab, "wpName", webServicePARAMETERMetadata.getWpName());
        print(tab, "wpMode", webServicePARAMETERMetadata.getWpMode());
        print(tab, "wpHeader", webServicePARAMETERMetadata.isWpHeader() ? "true" : "false");
        print(tab, "javaType", webServicePARAMETERMetadata.getJavaType());
        //print( "defaultName", webServicePARAMETERMetadata.getDefaultName());

        print("");
    }

    public static void print(BeehiveWsSOAPBindingInfo soapBindingInfo) {
        print(0, soapBindingInfo);
    }

    public static void print(int tab, BeehiveWsSOAPBindingInfo soapBindingInfo) {
        if (soapBindingInfo == null)
            return;
        print(tab, ">SOAPBindingInfo");
        print(tab, "style", soapBindingInfo.getStyle());
        print(tab, "use", soapBindingInfo.getUse());
        print(tab, "style", soapBindingInfo.getParameterStyle());
    }



    public static void print(int tab, String msg, Object data) {
        print(tab, msg + " : " + data);
    }

    public static void print(int tab, String msg) {
        print(TABS[tab] + msg);
    }

    public static void print(String msg) {
        System.out.println(msg);
    }

}
