/*
 * XmlBeanTypeMappingUtil.java
 * 
 * Copyright 2001-2004 The Apache Software Foundation.
 * 
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 * 
 * Original author: Jonathan Colwell
 */
package org.apache.beehive.wsm.util;

import java.lang.reflect.Method;

import javax.xml.namespace.QName;

import org.apache.xmlbeans.SchemaField;
import org.apache.xmlbeans.SchemaType;
import org.apache.xmlbeans.SchemaTypeLoader;
import org.apache.xmlbeans.XmlBeans;
import org.apache.xmlbeans.XmlObject;

/*******************************************************************************
 * 
 *
 * @author Jonathan Colwell
 */
public class XmlBeanTypeMappingUtil implements TypeMappingUtil {

    public QName registerType(Class cls) throws InvalidTypeMappingException {
       throw new InvalidTypeMappingException
           ("this implementation does not support type registration");
    }

    public QName registerType(Class cls, QName expectedType)
        throws InvalidTypeMappingException {
        
        throw new InvalidTypeMappingException
            ("this implementation does not support type registration");
    }

    public QName generateQName(Class type, String defaultNS) {

        if (XmlObject.class.isAssignableFrom(type)) {
            return XmlBeans.typeForClass(type).getName();
        }
        else {
            return null;
        }
    }

    public Class q2Class(QName qType) {

        SchemaTypeLoader stl = XmlBeans.getContextTypeLoader();
        SchemaType st = stl.findType(qType);
        if (st == null) {
            SchemaField sf = stl.findElement(qType);
            if (sf != null) {
                st = sf.getType();
            }
        }

        if (st != null) {
            Class xmlClass = st.getJavaClass();

            //String clName = xmlClass.getName();
            if (st.isBuiltinType()) {
                Method[] declared = xmlClass.getDeclaredMethods();
                Class natural =
                    scanDeclaredMethodsForViableReturnType(declared);
                if (natural != null) {
                    return natural;
                }
                else {
                    // NOTE jcolwell@bea.com 2004-Nov-12 --
                    // XmlString declares no methods

                    if (xmlClass.isInterface()) {
                        for (Class cl : xmlClass.getInterfaces()) {
                            natural = scanDeclaredMethodsForViableReturnType
                                (cl.getDeclaredMethods());
                            if (natural != null) {
                                return natural;
                            }
                        }
                    }
                    else {
                        declared = xmlClass.getSuperclass().getDeclaredMethods();
                        natural = scanDeclaredMethodsForViableReturnType
                            (declared);
                        if (natural != null) {
                            return natural;
                        }
                    }
                }
            }
            /*
            else if (st.isSimpleType()){
                System.out.println(clName + " is simple but not built in");
            }
            else {
                System.out.println(clName + " is NOT built in");
            }
            System.out.println("resulting class: " + xmlClass.getName());
            */
            return xmlClass;
        }
        else {
            // NOTE jcolwell@bea.com 2004-Nov-30 -- 
            // keep in mind that a real TMU based on a viable SOAP stack should
            // be used and this pure XmlBean implementation is just a fallback.
            System.out.println("no schematype found for " + qType);
            return Object.class;
        }
    }

    private Class scanDeclaredMethodsForViableReturnType
        (Method[] declared) {
        
        for (Method meth : declared) {
                
            Class returnType = meth.getReturnType();
            //System.out.println(returnType.getName());
            if (!returnType.equals(Void.TYPE)) {
                /*
                 * NOTE jcolwell@bea.com 2004-Nov-12 -- 
                 * built-in XmlBeans types may be of the following natural
                 * types:
                 * primitives, byte arrays, Strings, Calendars, BigIntegers
                 * and BigDecimals
                 */
                if (returnType.isArray()
                    || returnType.isPrimitive()
                    || returnType.equals(String.class)
                    || returnType.equals(QName.class)
                    || returnType.equals(java.util.Calendar.class)
                    || returnType.equals(java.math.BigDecimal.class)
                    || returnType.equals(java.math.BigInteger.class)) {

                    return returnType;
                }
            }
        }
        return null;
    }
}
