/*
 * 
 * Copyright 2001-2004 The Apache Software Foundation.
 * 
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 * 
 */
package org.apache.beehive.wsm.wsdl;

import java.io.IOException;
import java.io.InputStream;
import java.lang.reflect.Array;
import java.util.HashMap;
import java.util.Map;

import org.apache.xmlbeans.SchemaType;
import org.apache.xmlbeans.XmlException;
import org.apache.xmlbeans.XmlObject;

public class Utilities {

    public static <T extends XmlObject> T[] selectChildren(XmlObject parent,
            Class<T> childClass) throws IllegalAccessException,
            NoSuchFieldException {
        // retrieve the SchemaType from the static type field
        SchemaType st = (SchemaType) childClass.getField("type").get(null);
        XmlObject[] kids = parent.selectChildren(st.getDocumentElementName());
        T[] castKids = (T[]) Array.newInstance(childClass, kids.length);
        for (int j = 0; j < castKids.length; j++) {
            castKids[j] = childClass.cast(kids[j]);
        }
        return castKids;
    }

    /**
     * @param docType
     * @return
     * @throws XmlException
     * @throws IOException
     * @throws IllegalAccessException
     * @throws NoSuchFieldException
     */

    // Keep a cache of schemas that have been read.
    static Map<String, Schema> schemaCache = new HashMap<String, Schema>();
    public static Schema findtSchemaDocument(SchemaType docType)
            throws XmlException, IOException, IllegalAccessException,
            NoSuchFieldException {
        Schema mySchema = null;
        if (null != (mySchema = schemaCache.get(docType.getName()
                .getNamespaceURI()))) {
            return mySchema;
        }
        String schemaSrc = docType.getSourceName();
        InputStream stream = docType.getTypeSystem().getSourceAsStream(
                schemaSrc);

        if (null == stream) {
            throw new RuntimeException("WSDL file not found: " + schemaSrc);
        }
        if (schemaSrc.endsWith(".wsdl") || schemaSrc.endsWith(".WSDL")) {
            mySchema = new WSDLParser(stream).getSchema();
        } else {
            mySchema = new Schema(stream);
        }
        schemaCache.put(docType.getName().getNamespaceURI(), mySchema);
        return mySchema;
    }

}
