package org.apache.beehive.wsm.wsdl;

/*
 * 
 * Copyright 2001-2004 The Apache Software Foundation.
 * 
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 * 
 * 
 */

import java.io.IOException;
import java.io.InputStream;
import java.lang.reflect.Array;

import javax.xml.namespace.QName;

import org.xmlsoap.schemas.wsdl.DefinitionsDocument;
import org.xmlsoap.schemas.wsdl.TPort;
import org.xmlsoap.schemas.wsdl.TService;
import org.xmlsoap.schemas.wsdl.TTypes;
import org.xmlsoap.schemas.wsdl.soap.TAddress;

import org.apache.xmlbeans.SchemaType;
import org.apache.xmlbeans.XmlException;
import org.apache.xmlbeans.XmlObject;

import org.apache.xmlbeans.impl.xb.xsdschema.SchemaDocument;
import org.apache.xmlbeans.impl.xb.xsdschema.TopLevelComplexType;
import org.apache.xmlbeans.impl.xb.xsdschema.TopLevelElement;
import org.apache.xmlbeans.impl.xb.xsdschema.TopLevelSimpleType;

public class WSDLParser {

 
    InputStream stream;
    DefinitionsDocument defDoc;
    TTypes tt;
    Schema schema;
	TService[] services;
 
    /**
     * @param stream
     * @throws IOException
     * @throws XmlException
     * @throws NoSuchFieldException
     * @throws IllegalAccessException
     */
    public WSDLParser(InputStream stream) throws XmlException, IOException, IllegalAccessException, NoSuchFieldException {
        super();
        this.stream = stream;

        defDoc = DefinitionsDocument.Factory.parse(stream);
        tt = defDoc.getDefinitions().getTypesArray(0);
        schema = new Schema(tt);
        services = defDoc.getDefinitions().getServiceArray();
		 
    }
    /**
     * @return Returns the schema.
     */
    public Schema getSchema() {
        return schema;
    }
    
    public String getSoapAddressLocation() throws IllegalAccessException {
        String location = null;
        if (services.length > 0) {
            TPort port = services[0].getPortArray(0);
            if(null != port) {
                 try {
                    TAddress[] soapAddress = Utilities.selectChildren(port, TAddress.class);
                    if(soapAddress.length > 0) {
                        location = soapAddress[0].getLocation();
                    }
                } catch (NoSuchFieldException e) {
                    // just return null 
                }
            }
        }
         return location;
    }
	
	


}
